#ifndef STOK_C
#define STOK_C
typedef struct stok_s { char *delim, *a, val, restore; } stok_t;

#define STOK_FOREACH(CPTR_VAR, CPTR_DELIM, CPTR_STRING) \
        stok_t  CPTR_VAR##__Tok, \
               *CPTR_VAR##__tok = stok_make(&CPTR_VAR##__Tok, CPTR_DELIM, CPTR_STRING); \
        while ((CPTR_VAR = stok_next(CPTR_VAR##__tok)))

stok_t *stok_make(stok_t *t, char *delim, char *a) {
        if (!t && !(t = (stok_t *) malloc(sizeof(stok_t))))
                return t;                       /* no memory */
        t->delim   = delim;
        t->a       = a;
        t->val     = '\0';
        t->restore = delim[0] != delim[1];      /* duplicate => no restore */
        return t;
}

void stok_fix(stok_t *t) {                      /* restore saved val */
        if (t->val)
            *t->a = t->val;
}

char *stok_next(stok_t *t) {
        char *p;

        if (t->val) {
                if (t->restore)                 /* duplicate => leave NUL */
                        *t->a = t->val;         /* restore saved val */
                t->a++;                         /* eat last delimiter */
        } else if (!t->a)                       /* !t->val && !t->a ==> done */
                return NULL;                    /* last saved value was EOS */
        t->a += strspn(t->a, t->delim);         /* scan leading delimiters */
        if (!*t->a)
                return NULL;                    /* delimiters to EOS */
        p    = t->a;                            /* save start of token */
        t->a = strpbrk(p, t->delim);            /* advance 'a' to next delim */
        if (!t->a)                              /* End-Of-String */
                t->val = '\0';                  /* signal EOS next time */
        else {
                t->val = *t->a;                 /* save value */
                *t->a  = '\0';                  /* terminate token */
        }
        return p;
}

int stok_splitN(char *a, char *d, char **f, int N) {    /* overflow -> last */
        int      n;
        stok_t   Tok, *t = stok_make(&Tok, d, a);

        Tok.restore = 0;                                /* force leaving NUL */
        for (n = 0; n < N && (f[n] = stok_next(t)); n++)
                /**/;
        if (n == N && t->a)
                *t->a = t->val;
        return n;
}
#endif /* STOK_C */
